"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.markCursorInTemplate = exports.applySemanticRules = exports.getSemanticContext = exports.tryGetCursorHTMLContext = void 0;
const tslib_1 = require("tslib");
const node_html_parser_1 = tslib_1.__importDefault(require("node-html-parser"));
const CURSOR_TAG = 'ctx-cursor';
/**
 * Returns the DOM hierarchy at the cursor position up to the root of the text.
 * @param ctx - The completion context
 * @returns node names in order from cursor to root
 *
 * @remarks May return malformed result if the html markup is not valid. e.g. missing
 * closing tags.
 */
function tryGetCursorHTMLContext(ctx) {
    const cursorPos = ctx.context.toOffset(ctx.position);
    const markedTemplate = (0, exports.markCursorInTemplate)(ctx.context.text, cursorPos);
    const root = (0, node_html_parser_1.default)(markedTemplate);
    const cursorNode = root.querySelector(CURSOR_TAG);
    let node = cursorNode;
    const domList = [];
    while (node.parentNode !== null) {
        domList.push(node.rawTagName);
        node = node.parentNode;
    }
    domList.splice(0, 1);
    return domList;
}
exports.tryGetCursorHTMLContext = tryGetCursorHTMLContext;
/**
 * Returns a set of semantic contexts based on the html context.
 * @param htmlContext - The html context from `tryGetCursorHTMLContext`
 * @param designSystemPrefix - The prefix of the design system, used to match tag names
 * @returns A set of semantic contexts
 */
function getSemanticContext(htmlContext, designSystemPrefix) {
    var _a, _b;
    if ((_a = htmlContext[0]) === null || _a === void 0 ? void 0 : _a.includes(`${designSystemPrefix}-layout-item`)) {
        return ['DYNAMIC_LAYOUT_ITEM'];
    }
    if ((_b = htmlContext[0]) === null || _b === void 0 ? void 0 : _b.includes(`${designSystemPrefix}-layout`)) {
        return ['DYNAMIC_LAYOUT'];
    }
    return ['UNKNOWN'];
}
exports.getSemanticContext = getSemanticContext;
const MAX_SORTING = 9;
const PROMOTION_SORTING = 5;
/**
 * Applies any rules set on a snippet depending on the semantic context.
 * @param rules - The rules set on the snippet
 * @param semanticCtx - The semantic context
 * @returns The sort text for the snippet, or null if the snippet should not be shown. `0` is a snippet with the highest precedent, `9` is the lowest.
 */
function applySemanticRules(rules, semanticCtx, baseSorting) {
    let sorting = baseSorting;
    const appliedRules = rules
        .filter(({ semanticCtx: ruleContext }) => semanticCtx.includes(ruleContext))
        .map(({ rule }) => rule);
    if (appliedRules.includes('HIDE')) {
        return null;
    }
    if (appliedRules.includes('PROMOTION')) {
        sorting -= PROMOTION_SORTING;
    }
    return Math.min(Math.max(sorting, 0), MAX_SORTING).toString();
}
exports.applySemanticRules = applySemanticRules;
const markCursorInTemplate = (templateText, pos) => templateText.slice(0, pos) + `<${CURSOR_TAG}></${CURSOR_TAG}>` + templateText.slice(pos);
exports.markCursorInTemplate = markCursorInTemplate;
